/*  KRunning - Evaluation.h
 *  Copyright (C) 2002 by Andreas Scherf
 *  Begin Fre Mai 24 2002 write to scherfa@web.de if you found a bug.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
#ifndef EVALUATION_H
#define EVALUATION_H
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif
// QT Stuff
#include <qdatetime.h>
#include <qprogressbar.h>
// KDE STuff
#include <ktextbrowser.h>
#include <kpushbutton.h>
#include <kprinter.h>


#include "DBConnection.h"
#include "EvaluationGUI.h"
#include "Printing.h"

class classCountObj {
    public:
        classCountObj() : count(-1) {}
        classCountObj(const QString &cn,int c) : classname(cn),count(c) {}
        QString getClassName() const {return classname;}
        int getCount() const {return count;}
        classCountObj & operator++() {count++;return *this;}
    private :
        QString classname;
        int count;
};

class teamCountObj {
    public:
        teamCountObj() : count(-1) {}
        teamCountObj(int num,int c) : teamnumber(num),count(c) {}
        int getTeamNumber() const {return teamnumber;}
        int getCount() const {return count;}
        teamCountObj & operator++() {count++; return *this;}
    private :
        int teamnumber;
        int count;
};

class teamEvalObj {
    public:
        teamEvalObj() : count(-1) {}
        teamEvalObj(int num,int c) : teamnumber(num),count(c) {}  // find ...
        teamEvalObj(int num,int c,runnerObj &,timeObj &);
        void setTeamPlace(int p) {place=p;}
        bool addRunner(runnerObj &runner,timeObj &time);

        int getTeamPlace() const {return place;}
        int getTeamNumber() const {return teamnumber;}
        int getCount() const {return count;}
        teamEvalObj & operator++() {count++; return *this;}
        QTime getTimeSum() const {return timesum;}
        DBConnection::TimeList getTimeList() const {return times;}
        DBConnection::RunnerList getRunnerList() const {return runners;}
        int getTeamOrder() const {return teamorder;}

    private :
        int teamnumber;
        int teamorder;
        int count; // number of runners, ...
        int place;
        DBConnection::RunnerList runners;
        DBConnection::TimeList times;
        QTime timesum;
};
// This is for qBubbleSort and qHeapSort functions
extern bool operator<(teamEvalObj &,teamEvalObj &);

// For the find functions
extern bool operator==(teamCountObj ,teamCountObj);
extern bool operator==(classCountObj,classCountObj);
extern bool operator==(teamEvalObj,teamEvalObj);
extern bool operator==(teamObj,teamObj);

/** Evaluation
@version 0.7
*/
class Evaluation  : public EvaluationGUI {
        Q_OBJECT
    public:
        Evaluation (QWidget * parent = 0, const char *name = 0);
        virtual ~Evaluation();
        void setDefaultSettings();
        void print(KPrinter *printer);
        void save();
        static   QString getClassString (int yearOfBirth,const QString &sex);
        static   QString getClassString (int yearOfBirth,runnerObj::SexNum num);
        typedef QValueList<classCountObj> ClassCountList;
        typedef QValueList<teamCountObj> TeamCountList;
        // For the Team Evaluation
        typedef QValueList<teamEvalObj> TeamEvalList;
        QValueListIterator<teamEvalObj> findLast(TeamEvalList &,const teamEvalObj &);
    protected:

    protected slots:
        void slotShowTeamResults();
        void slotShowClassResults();
        void evaluateAllRunners();
        void evaluateClassResults(runnerObj::SexNum sex);
        void evaluateTeamResults(runnerObj::SexNum sex);
    private:
        QString resultText;
};

#endif // EVALUATION_H
