/*  KRunning - RunnerEdit.cpp
 *  Copyright (C) 2002 by Andreas Scherf
 *  Begin Mit Mai 22 2002 write to scherfa@web.de if you found a bug.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif


#include "RunnerEdit.h"

using namespace std;

RunnerEdit::RunnerEdit (QWidget * parent, const char *name) : RunnerEditGUI(parent,name,true) {
    lastRunnerNum=-1;
    connect((QObject*)StartNumEdit,SIGNAL(valueChanged(int)),this, SLOT(startNumberEntered()) );
    connect((QObject*)SexCombo,SIGNAL(activated(int)),this, SLOT(sexCharEntered(int)) );
    connect((QObject*)BirtdayInput,SIGNAL(valueChanged(int)),this,SLOT(sexCharEntered(int)));
    connect((QObject*)ChangeNumberButton,SIGNAL(pressed()),this,SLOT(changeStartnumber()));
    updateRunner=false;
}

RunnerEdit::~RunnerEdit() {
}

void RunnerEdit::startNumberEntered() {
    runnerObj runner;
    lastRunnerNum=StartNumEdit->value();
    runner=dbconn->getRunnerByNumber(lastRunnerNum);
    if (runner.runnernumber()!=-1) {
        NameEdit->setText(runner.runnersurname());
        PreNameEdit->setText(runner.runnerforename());
        BirtdayInput->setValue(runner.runnerbirthday());
        SexCombo->setCurrentItem(runner.runnersex());
        PLZEdit->setValue(runner.runnerplz());
        StreetEdit->setText(runner.runnerstreet());
        CityEdit->setText(runner.runnercity());
        EventCombo->setCurrentText(dbconn->getEventNameByNumber(runner.runnerevent()));
        TeamCombo->setCurrentText(dbconn->getTeamNameByNumber(runner.runnerevent()));
        CommentEdit->setText(runner.runnercomment());
        updateRunner=true;
    } else {
        NameEdit->setText("");
        PreNameEdit->setText("");
        StreetEdit->setText("");
        CityEdit->setText("");
        PLZEdit->setValue(0);
        BirtdayInput->setValue(1900);
        SexCombo->setCurrentItem(0);
        EventCombo->setCurrentItem(0);
        TeamCombo->setCurrentItem(0);
        CommentEdit->clear();
        updateRunner=false;
    }
    sexCharEntered(-1);
}

void RunnerEdit::sexCharEntered(int num) {   // Bad Name
    QString text=QString(i18n("Class"));
    text.append(" ");
    text.append(Evaluation::getClassString(BirtdayInput->value(),SexCombo->text(SexCombo->currentItem())));
    ClassLabel->setText(text);
}

void RunnerEdit::setDefaultSettings() {

    PLZEdit->setMinValue(0); // PLZ
    StartNumEdit->setMinValue(0); // StartNumber

    if (lastRunnerNum==-1) {
        StartNumEdit->setValue(0);
    } else {
        StartNumEdit->setValue(lastRunnerNum);
    }

    StartNumEdit->setFocus();
    TeamCombo->clear();
    TeamCombo->setDuplicatesEnabled ( FALSE );
    TeamCombo->insertStringList(dbconn->getTeamNameList() ,-1);
    EventCombo->clear();
    EventCombo->insertStringList(dbconn->getEventNameList(),-1);
    SexCombo->clear();
    SexCombo->insertItem(i18n("Male"),-1);
    SexCombo->insertItem(i18n("Female"),-1);
    BirtdayInput->setMinValue(1900); // Birthday
    if (prefs->getACRunnerNames()==true) {
        // Turn Off didn work correctly
        KCompletion *compNames = NameEdit->completionObject();
        KCompletion *compForeNames = PreNameEdit->completionObject();
        compNames->insertItems (dbconn->getNameList(true) );
        compForeNames->insertItems(dbconn->getNameList(false));
        compNames->setIgnoreCase(true);
        compForeNames->setIgnoreCase(true);
        // Connect to the return pressed signal - optional
        connect(NameEdit,SIGNAL(returnPressed(const QString&)),compNames,SLOT(addItem(const QString&)));
        connect(PreNameEdit,SIGNAL(returnPressed(const QString&)),compForeNames,SLOT(addItem(const QString&)));
    }
}

void RunnerEdit::keyPressEvent (QKeyEvent * k) {
    switch (k->key ()) {
    case Qt::Key_Return :
        cerr << "Return pressed" << endl;
        break;
    case Key_F12 :
        cerr << "F12 pressed" << endl;
        checkAllEntrys();
        insertNewRunner();
        setupNewRunner();
        break;
    default:
        k->ignore();
    }
}

bool RunnerEdit::checkAllEntrys() {
    if (NameEdit->text().length() < 1 || PreNameEdit->text().length() < 1 || BirtdayInput->value() < 1900) {
        KMessageBox::warningContinueCancel (this,i18n("Enter a valid name please"),"Warning ...");
    }

    return TRUE;
}

void RunnerEdit::insertNewRunner() {
    if (prefs->getEventFromStartNumber() && !updateRunner) {
        if (dbconn->getEventByStartnumber(  StartNumEdit->value() ).getEventNumber()!=-1)
            EventCombo->setCurrentText( dbconn->getEventByStartnumber(  StartNumEdit->value() ).getEventName());
    }
    lastEventNum=EventCombo->currentItem();

    runnerObj *runner=new runnerObj(StartNumEdit->value(),NameEdit->text(),PreNameEdit->text(),BirtdayInput->value(),
                                    ((runnerObj::SexNum)SexCombo->currentItem()), dbconn->getEventnumberByName(EventCombo->currentText()),dbconn->getTeamnumberByName(TeamCombo->currentText()),
                                    StreetEdit->text(),CityEdit->text(),PLZEdit->value(),KGlobal::locale()->country()/*Change this !!*/,"",runnerObj::Created,CommentEdit->text(),-1,-1,-1,-1,-1);

    runner->save( updateRunner );
    delete runner;
}


void RunnerEdit::setupNewRunner() {
    StartNumEdit->setValue(StartNumEdit->value()+1);
    NameEdit->setFocus();
    if (prefs->getRemeberEvent()) {
        EventCombo->setCurrentItem(lastEventNum);
    }
}

void RunnerEdit::changeStartnumber() {
    if (lastRunnerNum == -1) return;
    runnerObj *runner=new runnerObj(StartNumEdit->value(),NameEdit->text(),PreNameEdit->text(),BirtdayInput->value(),
                                    ((runnerObj::SexNum)SexCombo->currentItem()), dbconn->getEventnumberByName(EventCombo->currentText()),dbconn->getTeamnumberByName(TeamCombo->currentText()),
                                    StreetEdit->text(),CityEdit->text(),PLZEdit->value(),KGlobal::locale()->country()/*Change this !!*/,"",runnerObj::Created,CommentEdit->text(),-1,-1,-1,-1,-1);

    ChangeNumberDialog * dlg=new ChangeNumberDialog(this,"ChangeNumberDialog");
    dlg->setOldNumber(lastRunnerNum);
    dlg->setName(QString("%1, %2").arg(NameEdit->text()).arg(PreNameEdit->text()));
    if (dlg->exec()==KDialog::Accepted) {
        runner->setNewRunnerNumber(dlg->getNewNumber());
        runner->save( updateRunner );
    }

    delete runner;
    delete dlg;
}

//-----------


ChangeNumberDialog::ChangeNumberDialog( QWidget* parent,  const char* name )
        :   KDialogBase( Plain, i18n("Change Startnumber" ),  Ok|Cancel, Ok, parent,name, true,true)  {

    QFrame* privateLayoutWidget = plainPage();
    Layout4 = new QVBoxLayout( privateLayoutWidget, 0, 6, "Layout4");

    Layout2_2 = new QHBoxLayout( 0, 0, 6, "Layout2_2");

    TextLabel2_2 = new QLabel( privateLayoutWidget, "TextLabel2_2" );
    TextLabel2_2->setText( tr2i18n( "Old Number :" ) );
    Layout2_2->addWidget( TextLabel2_2 );

    OldNumberLabel = new QLabel( privateLayoutWidget, "OldNumberLabel" );

    Layout2_2->addWidget( OldNumberLabel );
    Layout4->addLayout( Layout2_2 );

    Layout3_3 = new QHBoxLayout( 0, 0, 6, "Layout3_3");

    TextLabel4_2 = new QLabel( privateLayoutWidget, "TextLabel4_2" );
    TextLabel4_2->setText( tr2i18n( "Name :" ) );
    Layout3_3->addWidget( TextLabel4_2 );

    NameLabel = new QLabel( privateLayoutWidget, "NameLabel" );
    Layout3_3->addWidget( NameLabel );
    Layout4->addLayout( Layout3_3 );

    Layout3_4 = new QHBoxLayout( 0, 0, 6, "Layout3_4");

    TextLabel3_2 = new QLabel( privateLayoutWidget, "TextLabel3_2" );
    TextLabel3_2->setText( tr2i18n( "New Number :" ) );
    Layout3_4->addWidget( TextLabel3_2 );

    NewNumberInput= new KIntNumInput( privateLayoutWidget, "NewNumberInput" );
    NewNumberInput->setMinValue(0);
    Layout3_4->addWidget( NewNumberInput);

    pixLabel = new QLabel( privateLayoutWidget, "PixmapLabel1" );
    pixLabel ->setFrameShape( QLabel::NoFrame );
    pixLabel ->setMargin( 1 );
    pixLabel ->setPixmap(KGlobal::iconLoader()->loadIcon( "button_cancel", KIcon::NoGroup, KIcon::SizeSmall ) );
    Layout3_4->addWidget( pixLabel );
    Layout4->addLayout( Layout3_4 );
    connect((QObject*)NewNumberInput,SIGNAL(valueChanged(int)),this, SLOT(startNumberChanged()) );
    enableButtonOK(false);
}

/*
 *  Destroys the object and frees any allocated resources
 */
ChangeNumberDialog::~ChangeNumberDialog() {

}

void ChangeNumberDialog::setOldNumber(int num) {
    OldNumberLabel->setText( QString::number(num) );
}
void ChangeNumberDialog::setName(const QString &name) {
    NameLabel->setText( name );
}

int ChangeNumberDialog::getNewNumber() {
    return (       NewNumberInput->value() );
}

void ChangeNumberDialog::startNumberChanged() {
    runnerObj runner;
    runner=dbconn->getRunnerByNumber(  NewNumberInput->value());
    if (runner.runnernumber()==-1) {
        pixLabel ->setPixmap(KGlobal::iconLoader()->loadIcon( "button_ok", KIcon::NoGroup, KIcon::SizeSmall ) );
        enableButtonOK(true);
    } else {
        pixLabel ->setPixmap(KGlobal::iconLoader()->loadIcon( "button_cancel", KIcon::NoGroup, KIcon::SizeSmall ) );
        enableButtonOK(false);
    }
}
